import torch
import os
from pyfasta import Fasta
import numpy as np
import pandas as pd
import time
import json

acgt2num = {'A': 0,
            'C': 1,
            'G': 2,
            'T': 3}

def seq2mat(seq):
    seq = seq.upper()
    h = 4
    w = len(seq)
    mat = np.zeros((h, w), dtype=bool)  # True or false in mat
    for i in range(w):
        if seq[i] != 'N':
            mat[acgt2num[seq[i]], i] = 1.
    return mat

genome = Fasta('hg38.fa')

def get_tf_motif_match():
    """
    Return:
        Two items
            First item: a list of TF(gene) name (711 TFs).
            Second item: dic mapping motif name to tf (gene) name
    """
    from scipy.io import loadmat
    motif_info = loadmat('./motifdata/MotifMatch_human_rmdup.mat') 
    motif_list = [item.split('\t')[1] for item in open('./motifdata/all_motif_rmdup.motif').readlines() \
        if item.startswith('>') ]
    motif2tf = {}
    for each in motif_info['Match2']:
        motif = each[0][0]
        tf = each[1][0]
        if motif not in motif2tf.keys():
            motif2tf[motif] = [tf]
        else:
            motif2tf[motif]+=[tf]
    motif_list_filtered = [item for item in motif2tf.keys() if item in motif_list]
    tf_list = []
    for each in motif_list_filtered:
        tf_list += motif2tf[each]
    tf_list = list(set(tf_list))
    tf_list.sort()

    isContain = {item : True if item in motif_list_filtered else False for item in motif_list}
    
    return tf_list, {item : motif2tf[item] for item in motif_list_filtered}, isContain

def model_predict(model,seq_embeds,tf_feature):
    seq_embeds = torch.from_numpy(seq_embeds)
    tf_feature = np.pad(tf_feature,((0, 0), (0, 1)),'constant',constant_values = (0,0))
    tf_feature = np.expand_dims(tf_feature,axis=0)
    tf_feature = torch.from_numpy(tf_feature)
    seq_embeds = seq_embeds.type(torch.FloatTensor)
    tf_feature = tf_feature.type(torch.FloatTensor)
    seq_embeds = seq_embeds.to('cuda')
    tf_feature = tf_feature.to('cuda')
    signals = model(seq_embeds,tf_feature)
    np_signals = signals.cpu().detach().numpy()
    return np_signals


def load_weights(model,path):
    model.load_state_dict(torch.load('pretrainModel/model.ckpt',map_location='cuda:0')['state_dict'])
    model = model.eval()
    model = model.cuda()
    return model

def get_motif_score(tf_list, motif2tf, isContain, region_file, motifscan_file, save = True):
    """parse the motifscan results by homer tool.
    Args:
        region_file: path to bed file that was used for motif scanning
        motifscan_file: path to the motifscan file from Homer tool
    """
    regions = ['_'.join(item.split('\t')[:4]) for item in open(region_file).readlines()]
    motifscore = np.empty((len(regions), len(tf_list)), dtype = 'float32')
    count = 0
    f_motifscan = open(motifscan_file,'r')
    #skip header
    line = f_motifscan.readline()
    print(line)
    line = f_motifscan.readline()
    print(line)
    while line != '':
        motif = line.split('\t')[3]
        if isContain[motif]:
            region_id = line.split('\t')[0]
            score = float(line.split('\t')[-1].rstrip())
            for tf in motif2tf[motif]:
                previous = motifscore[int(region_id)-1][tf_list.index(tf)]
                motifscore[int(region_id)-1][tf_list.index(tf)] = max(score, previous)
        line = f_motifscan.readline()
        count += 1
    f_motifscan.close()
    np.save('motifscore.npy', motifscore)
def get_motifscore(region,tf_list,motif2tf):
#     for chr in keys:
    chrom,start,end = region[0], int(region[1]),int(region[2])
    if start < 0 or end > len(genome[chrom]):
        return np.zeros((1000,711))
    scan_file = pd.read_csv('/work/gaozijing/EpiGePT_revision/data/motifscore_allgenome/files/%s_scan_tmp.csv'%(chrom),sep='\t')
    location_start = int(start/1000)
    location_end = int(end/1000)
    with open("/work/gaozijing/EpiGePT_revision/data/motifscore_allgenome/files/%s_location_map.json"%(chrom), encoding="utf-8") as f:
        location_map = json.load(f)
    left_side = location_map[str(location_start)]
    right_side = location_map[str(location_end)]
    if right_side < left_side:
        scan_file_part = scan_file.iloc[right_side:left_side]
        # 1000 tokens
        motifscore = np.zeros((1000,711))
        for i in range(len(scan_file_part)):
            motif = scan_file_part['Motif Name'].iloc[i]
            PositionID = scan_file_part['PositionID'].iloc[i]
            Offset = scan_file_part['Offset'].iloc[i]
            position = int(PositionID) * 1000 + int(Offset)
            region_loc = int((position - start)//128)
            if region_loc > 999 :
                continue
            score = scan_file_part['MotifScore'].iloc[i]
            for tf in motif2tf[motif]:
                previous = motifscore[region_loc,tf_list.index(tf)]
                motifscore[region_loc,tf_list.index(tf)] = max(score, previous)
    else:
        # TF motif score set as 0
        motifscore = np.zeros((1000,711))
    return motifscore